"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkEntity = exports.throwErr = exports.UType = exports.setup = exports.PAGE_SIZE = exports.ShowSql = exports.NONE_PARAM = exports.isSupport = void 0;
const lodash_1 = __importDefault(require("lodash"));
const typebox_1 = require("@sinclair/typebox");
const value_1 = require("@sinclair/typebox/value");
const Providers_1 = require("./Providers");
const SUPPORT_MAP = new Map([
    ['string', new Set(['Min', 'MinThan', 'Max', 'MaxThan', 'Like', 'Likel', 'Liker', 'Not', 'IsNull', 'NotNull', 'IsDistinct', 'NotDistinct', '>', '>=', '<', '<=', '=', '!=', '<>'])],
    ['number', new Set(['Min', 'MinThan', 'Max', 'MaxThan', 'Bt', 'Not', 'IsNull', 'NotNull', 'IsDistinct', 'NotDistinct', '>', '>=', '<', '<=', '=', '!=', '<>'])],
    ['boolean', new Set(['Min', 'Max', 'Not', 'IsNull', 'NotNull', '>', '<', '=', '!=', '<>'])],
    ['date', new Set(['Min', 'MinThan', 'Max', 'MaxThan', 'MinH', 'MinD', 'MinM', 'MaxH', 'MaxD', 'MaxM', 'Bt', 'BtD', 'BtY', 'BtM', 'Not', 'IsNull', 'NotNull', '>', '>=', '<', '<=', '=', '!=', '<>'])],
]);
const isSupport = (type, suffix) => {
    let set = SUPPORT_MAP.has(type) ? SUPPORT_MAP.get(type) : SUPPORT_MAP.get('string');
    return set.has(suffix);
};
exports.isSupport = isSupport;
exports.NONE_PARAM = new Set(['IsNull', 'NotNull', 'IsDistinct', 'NotDistinct']);
var STRICT_QUERY = false;
var STRICT_ENTITY = false;
var ShowSql = (str, param) => { };
exports.ShowSql = ShowSql;
exports.PAGE_SIZE = 10;
const setup = (settings) => {
    if (lodash_1.default.isArray(settings.provider)) {
        (0, Providers_1.setProvider)(settings.provider[0], settings.provider[1]);
    }
    else if (lodash_1.default.isFunction(settings.provider)) {
        (0, Providers_1.setProvider)('pg', settings.provider);
    }
    if (settings.strict) {
        if (lodash_1.default.isBoolean(settings.strict)) {
            STRICT_QUERY = true;
            STRICT_ENTITY = true;
        }
        else {
            if (settings.strict.entity)
                STRICT_ENTITY = true;
            if (settings.strict.query)
                STRICT_QUERY = true;
        }
    }
    if (settings.pageSize)
        exports.PAGE_SIZE = settings.pageSize;
    if (settings.showSQL && lodash_1.default.isFunction(settings.showSQL))
        exports.ShowSql = settings.showSQL;
};
exports.setup = setup;
exports.UType = {
    Table: (properties) => typebox_1.Type.Partial(typebox_1.Type.Object(properties)),
    Number: (options) => typebox_1.Type.Number(options),
    String: (options) => typebox_1.Type.String(options),
    Date: (options) => typebox_1.Type.Date(options),
    Boolean: (options) => typebox_1.Type.Boolean(options),
    Integer: (options) => typebox_1.Type.Integer(options),
};
const throwErr = (err, message) => {
    if (err.length == 0)
        return;
    if (!STRICT_QUERY) {
        console.error(message + '\n      ' + err.join('\n      '));
        return;
    }
    throw new Error(message ? message : err[0], { cause: err.join('\n') });
};
exports.throwErr = throwErr;
const checkEntity = (T, val) => {
    if (!STRICT_ENTITY) {
        return;
    }
    const result = value_1.Value.Errors(T, val);
    let err = result.next();
    if (err) {
        throw new Error('Entity Has Some value');
    }
};
exports.checkEntity = checkEntity;
