"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.View = void 0;
const lodash_1 = __importDefault(require("lodash"));
const Util_1 = require("../../base/Util");
const BaseQuery_1 = require("./BaseQuery");
const ES_MAX_SIZE = 10000;
const executor_1 = require("../query/executor");
class View extends BaseQuery_1.BaseQuery {
    all() {
        const { _CONFIG: { BASE_QUERY } } = this;
        return executor_1.executor.query(this.getClient(), this._index, Object.assign(Object.assign({}, BASE_QUERY), { from: 0, size: ES_MAX_SIZE }));
    }
    /**
     * Exec A Senctence
    */
    sql(request) {
        return executor_1.executor.query.call(this, this.getClient(), (request && request.index) ? request.index : this._index, request);
    }
    /**
     * @param tableName Data table name, "${schemaName}.${tableName}"
     *  "${schemaName}." can be ignore with the default search_path.
     * @param schema The Object Schema, oor will not validate the value
     * @param options (Table/View) Options
     *
    */
    constructor(indexName, schema, options) {
        super();
        this._CONFIG = {
            key: null,
            mark: null,
            pageSize: Util_1.PAGE_SIZE,
            FIELD_MAP: new Map(),
            WHERE_FIX: ['', ' WHERE '],
            BASE_QUERY: {},
            // globalCondition: [] as WhereItem[]
        };
        this._QUERY_CACHE = new Map();
        this._index = indexName;
        this._CONFIG.BASE_QUERY.index = indexName;
        this._CONFIG.BASE_QUERY.from = 0;
        this._CONFIG.BASE_QUERY._source_excludes = [];
        this._CONFIG.FIELD_MAP = new Map();
        var WHERE = [];
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            let column = properties.column || field;
            this._CONFIG.FIELD_MAP.set(field, properties);
            if (lodash_1.default.has(properties, 'delMark') && properties.delMark != null) {
                this._CONFIG.mark = { [column]: properties.delMark };
                WHERE.push({ field: column, value: properties.delMark, condition: '!=' });
            }
            if (properties.ignore) {
                this._CONFIG.BASE_QUERY._source_excludes.push(field);
                return;
            }
            // if (properties.column) {
            //     fields_get.push(`"${properties.column}" AS "${field}"`);
            //     fields_query.push(`"${properties.column}" AS "${field}"`);
            // } else {
            //     fields_get.push('"' + field + '"');
            //     if (properties.ignore === true) {
            //         return;
            //     }
            //     fields_query.push('"' + field + '"');
            // }
        });
        if (this._CONFIG.BASE_QUERY._source_excludes.length == 0)
            lodash_1.default.unset(this._CONFIG, '_source_excludes');
        if (options == null)
            return;
        if (options.sortOrder)
            this._CONFIG.BASE_QUERY.sort = { [options.sortOrder]: options.sortBy ? options.sortBy : 'desc' };
        if (options.pageSize)
            this._CONFIG.pageSize = options.pageSize;
        if (options.globalCondition && options.globalCondition.length) {
            // WHERE = WHERE.concat(options.globalCondition)
        }
        ;
        if (options.key) {
            this._CONFIG.key = options.key;
            if (this._CONFIG.BASE_QUERY.sort == null) {
                this._CONFIG.BASE_QUERY.sort = { [options.key]: options.sortBy ? options.sortBy : 'desc' };
            }
        }
    }
    ;
}
exports.View = View;
