"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.where = void 0;
const lodash_1 = __importDefault(require("lodash"));
const Util_1 = require("../../base/Util");
// https://www.elastic.co/guide/en/elasticsearch/reference/8.5/query-dsl.html
// COMING 
// 'Min', 'MinThan', 'Max', 'MaxThan',                 // commom  > , >= , <  ,  <=
// 'MinH', 'MinD', 'MinM', 'MaxH', 'MaxD', 'MaxM',     // Only Date Hour / Day / Month
// 'Like', 'Likel', 'Liker',                           // Only String  like leftlike rightlike
// 'Bt', 'BtD', 'BtY', 'BtM',                          // BETWEEN, support Number/Date ,'BtY', 'BtM', 'BtD' Only  Spport Date
// 'Not',                                              // != or <>
// 'IsNull', 'NotNull',                                // isNull or Not NULL           This Suffix will avoid value
// 'IsDistinct', 'NotDistinct',                        // isDistinct or Not Distinct   This Suffix will avoid value
// '>', '>=', '<', '<=', '=', '!=', '<>'    
const OPERATION_MAP = new Map([
    ['Min', 'gt'],
    ['MinThan', 'gte'],
    ['Max', 'lt'],
    ['MaxThan', 'lte'],
    ['MinH', 'gte'],
    ['MinD', 'gte'],
    ['MinM', 'gte'],
    ['MaxH', 'lte'],
    ['MaxD', 'lte'],
    ['MaxM', 'lte'],
    ['Like', 'wildcard'],
    ['Likel', 'prefix'],
    ['Liker', 'wildcard'],
    ['Bt', 'between'],
    ['BtD', 'between'],
    ['BtY', 'between'],
    ['BtM', 'between'],
    // ['Not', 'Must Not'],
    ['IsNull', 'exist'],
    // ['NotNull', 'exist'],
    // ['IsDistinct', 'gt'],
    // ['NotDistinct', 'gt'],
    ['>', 'gt'],
    ['>=', 'gte'],
    ['<', 'lt'],
    ['<=', 'lte'],
    ['=', 'term'],
    ['!=', 'gt'],
    ['<>', 'gt'],
]);
const RANGE_MAP = new Map([
    ['Min', 'gt'],
    ['MinThan', 'gte'],
    ['Max', 'lt'],
    ['MaxThan', 'lte'],
    ['MinH', 'gte'],
    ['MinD', 'gte'],
    ['MinM', 'gte'],
    ['MaxH', 'lte'],
    ['MaxD', 'lte'],
    ['MaxM', 'lte'],
    // ['Like', 'wildcard'],
    // ['Likel', 'prefix'],
    // ['Liker', 'wildcard'],
    // ['Bt', 'between'],
    // ['BtD', 'between'],
    // ['BtY', 'between'],
    // ['BtM', 'between'],
    // ['Not', 'Must Not'],
    // ['IsNull', 'exist'],
    // ['NotNull', 'exist'],
    // ['IsDistinct', 'gt'],
    // ['NotDistinct', 'gt'],
    ['>', 'gt'],
    ['>=', 'gte'],
    ['<', 'lt'],
    ['<=', 'lte'],
    // ['=', 'term'],
    ['!=', 'gt'],
    ['<>', 'gt'],
]);
const TERMS = "{\"terms\":{\"field\":\"%s\",\"size\":%d,\"min_doc_count\":1,\"shard_min_doc_count\":0,\"show_term_doc_count_error\":false,\"order\":[{\"_count\":\"desc\"}]}}";
const whereText = (item, query, err) => {
    if (RANGE_MAP.has(item.fn)) {
        err.push(`${item.column}/(${item.type}) not support method ${item.fn}`);
        return;
    }
    let action = OPERATION_MAP.get(item.fn);
    switch (action) {
        case 'wildcard':
            query.push({ wildcard: { [item.column]: item.value + '' } });
            break;
        case 'term':
            query.push({ term: { [item.column]: item.value + '' } });
    }
    // query.push({ [action]: {} })
    // query.push({
    //     range: {
    //         [item.column]: {
    //             gte: item.value,
    //             lte: item.value,
    //         }
    //     }
    // })
};
const ItemToWhere = (whereItem, query, err) => {
    let item = Object.assign(Object.assign({}, whereItem), { fn: whereItem.fn ? whereItem.fn : '=', type: whereItem.type ? whereItem.type : 'string' });
    if (!(0, Util_1.isSupport)(item.type, item.fn)) {
        err.push(`${item.column}/(${item.type}) not support method ${item.fn}`);
        return;
    }
    switch (item.type) {
        case 'number':
            // whereNumber(item, pos, err);
            return;
        case 'string':
            // query.term = {}
            // query.push({
            //     wildcard: {
            //     }
            // })
            // query.push({})
            // (query.filter).push({ term: { 'd': {} } })
            whereText(item, query, err);
            return;
        case 'boolean':
            // whereBoolean(item, pos, err);
            return;
        case 'date':
            // whereDate(item, pos, err);
            return;
        default:
            // whereText(item, pos, err);
            return;
    }
};
const ConditionToWhere = (root, query, err) => {
    for (let group of root.items) {
        if (lodash_1.default.has(group, 'link')) {
            let children = [];
            ConditionToWhere(group, children, err);
            if (children.length == 0) {
                continue;
            }
            if (children.length == 1) {
                query.push(children[0]);
            }
            if (root.link == 'OR') {
                query.push({ bool: { should: children } });
            }
            query.push({ bool: { filter: children } });
        }
        else {
            ItemToWhere(group, query, err);
        }
    }
};
const where = (condition) => {
    let root = lodash_1.default.isArray(condition) ? { link: 'AND', items: condition } : condition;
    let query = [];
    let err = [];
    ConditionToWhere(root, query, err);
    (0, Util_1.throwErr)(err, 'Some DSL Error Occur');
    if (query.length == 0) {
        return { match_all: {} };
    }
    if (query.length == 1) {
        return { constant_score: { filter: query[0] } };
    }
    if (root.link == 'OR') {
        return { constant_score: { filter: { bool: { should: query } } } };
    }
    return { constant_score: { filter: { bool: { filter: query, should: [] } } } };
};
exports.where = where;
