"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setup = exports.Table = exports.View = exports.UType = void 0;
const lodash_1 = __importDefault(require("lodash"));
const typebox_1 = require("@sinclair/typebox");
const Util_1 = require("../base/Util");
const BaseView_1 = require("../base/BaseView");
const BaseTable_1 = require("../base/BaseTable");
const basic_1 = require("./basic");
const where_1 = require("./where");
const executor_1 = require("./executor");
// Export Some useful global apis/types.
var Util_2 = require("../base/Util");
Object.defineProperty(exports, "UType", { enumerable: true, get: function () { return Util_2.UType; } });
const PG = { select: basic_1.select, count: basic_1.count, insert: basic_1.insert, delete: basic_1.del, update: basic_1.update, where: where_1.where, orderBy: basic_1.orderBy, limit: basic_1.limit, byField: basic_1.byField, };
const fixWhere = (FIELD_MAP, extra) => {
    let ITEMS = [];
    let ctf = new Map();
    const convert = (kind, value) => {
        switch (kind) {
            case 'Boolean':
                return value;
            case 'Number':
                return parseFloat(value);
            case 'Integer':
                return parseInt(value);
            default:
                return value + '';
        }
    };
    for (let [key, val] of FIELD_MAP) {
        if (lodash_1.default.has(val, 'delMark')) {
            ITEMS.push({ column: (val.column || key), fn: '<>', value: convert(val[typebox_1.Kind], val.delMark) });
        }
        ctf.set((val.column || key), key);
    }
    extra.map(item => {
        // inner usage field
        // @ts-ignore
        let schema = FIELD_MAP.get(item.field) || FIELD_MAP.get(ctf.get(item.field));
        if (schema == null)
            return;
        ITEMS.push(Object.assign(Object.assign({}, item), { value: convert(schema[typebox_1.Kind], item.value) }));
    });
    if (ITEMS.length == 0)
        return ['', ' WHERE '];
    let [SQL, PARAM] = (0, where_1.where)(ITEMS);
    if (SQL.length == 0)
        return ['', ' WHERE '];
    PARAM.map((item, i) => {
        SQL = SQL.replaceAll(`$${i + 1}`, lodash_1.default.isNumber(item) ? (item + '') : `'${item}'`);
    });
    return [' WHERE ' + SQL, ' AND '];
};
class View extends BaseView_1.BaseView {
    constructor(tableName, schema, options) {
        super(tableName, schema, options);
        this._DB_TYPE = 'pg';
        this._BUILDER = PG;
        this._EXECUTOR = executor_1.executor;
        this._CONFIG.WHERE_FIX = fixWhere(this._CONFIG.FIELD_MAP, (options && options.globalCondition) ? options.globalCondition : []);
    }
    sql(...args) {
        return super.sql(...args);
    }
}
exports.View = View;
class Table extends BaseTable_1.BaseTable {
    constructor(tableName, schema, options) {
        super(tableName, schema, options);
        this._DB_TYPE = 'pg';
        this._BUILDER = PG;
        this._EXECUTOR = executor_1.executor;
        this._CONFIG.WHERE_FIX = fixWhere(this._CONFIG.FIELD_MAP, (options && options.globalCondition) ? options.globalCondition : []);
    }
    sql(...args) {
        return super.sql(...args);
    }
}
exports.Table = Table;
const setup = (settings) => (0, Util_1.setup)(Object.assign(Object.assign({}, settings), { provider: ['pg', settings.provider] }));
exports.setup = setup;
